import { CanceledPromiseError, ClusteringHelper, Debouncer, waitFor, waitUntil } from "@fluid-topics/ft-wc-utils";
import { createReaderStore } from "./redux";
import { FtOfficialReaderService } from "./utils/FtOfficialReaderService";
import { ftAppInfoStore } from "@fluid-topics/ft-app-context";
export class FtReaderStateManager {
    static build(id, serviceProvider) {
        return new FtReaderStateManager(createReaderStore(id.trim() || "context"), serviceProvider !== null && serviceProvider !== void 0 ? serviceProvider : FtOfficialReaderService.build);
    }
    loaded() {
        return waitUntil(() => this._loaded);
    }
    constructor(store, serviceProvider, clusteringHelperProvider) {
        this.store = store;
        this.clusteringHelperProvider = clusteringHelperProvider;
        this.metadataForSwitchToRelatives = new Set();
        this._loaded = false;
        this.loadDebouncer = new Debouncer(10);
        this.initService(serviceProvider);
    }
    async initService(serviceProvider) {
        this.service = await serviceProvider();
        this.service.mapId = this.store.getState().mapId;
        return this.reload();
    }
    async setMapId(mapId) {
        if (mapId !== this.store.getState().mapId) {
            this.store.actions.mapId(mapId);
            if (this.service) {
                this.service.mapId = mapId;
            }
            await this.reload();
        }
    }
    setRenderedTopics(tocIds) {
        this.store.actions.renderedTopics(tocIds);
    }
    setVisibleTopics(tocIds) {
        this.store.actions.visibleTopics(tocIds);
    }
    async awaitService() {
        return waitFor(() => this.service);
    }
    async navigateToTopic(tocId, section, pageNumber) {
        await this.loaded();
        const service = await this.awaitService();
        const page = await (pageNumber != null ? service.getPage(pageNumber) : service.getPageByTocId(tocId !== null && tocId !== void 0 ? tocId : "root"));
        this.store.actions.currentPage(page);
        this.store.actions.scrollTarget({
            tocId: this.resolveValidNonRootTocId(tocId, page),
            section
        });
    }
    resolveValidNonRootTocId(tocId, page) {
        var _a, _b, _c;
        let nonRootTocId = (_a = (tocId === "root" ? undefined : tocId)) !== null && _a !== void 0 ? _a : page.rootTocId;
        if (!page.topics.includes(nonRootTocId)) {
            return (_c = (_b = page.topics[0]) !== null && _b !== void 0 ? _b : page.hiddenTopics[0]) !== null && _c !== void 0 ? _c : "root";
        }
        return nonRootTocId;
    }
    async navigateToPage(pageNumber) {
        await this.loaded();
        const service = await this.awaitService();
        let page = await service.getPage(pageNumber);
        this.store.actions.currentPage(page);
    }
    scrollDone() {
        this.store.actions.scrollTarget({});
    }
    async load() {
        var _a;
        if (((_a = this.service) === null || _a === void 0 ? void 0 : _a.mapId) && this.store.getState().map == null) {
            await this.loadDebouncer.run(() => this.fetchData());
        }
    }
    clear() {
        var _a;
        this._loaded = false;
        this.store.actions.configuration(undefined);
        this.store.actions.map(undefined);
        this.store.actions.toc(undefined);
        this.store.actions.pagesToc(undefined);
        this.store.actions.currentPage(undefined);
        this.store.actions.searchInDocumentQuery("");
        this.store.actions.relatives(undefined);
        this.store.actions.renderedTopics([]);
        this.store.actions.visibleTopics([]);
        (_a = this.service) === null || _a === void 0 ? void 0 : _a.clear();
        this.clusteringHelper = undefined;
    }
    reload() {
        this.clear();
        return this.load();
    }
    async fetchData() {
        var _a, _b, _c, _d, _e;
        try {
            await Promise.all([
                (_a = this.service) === null || _a === void 0 ? void 0 : _a.getConfiguration().then(conf => this.store.actions.configuration(conf)),
                (_b = this.service) === null || _b === void 0 ? void 0 : _b.getMap().then(map => this.store.actions.map(map)),
                (_c = this.service) === null || _c === void 0 ? void 0 : _c.getToc().then(toc => this.store.actions.toc(toc)),
                (_d = this.service) === null || _d === void 0 ? void 0 : _d.getPagesToc().then(pagesToc => this.store.actions.pagesToc(pagesToc)),
                (_e = this.service) === null || _e === void 0 ? void 0 : _e.getPaginationConfiguration().then(conf => this.store.actions.paginationConfiguration(conf)),
            ]);
            if (this.metadataForSwitchToRelatives.size > 0) {
                this.fetchRelatives();
            }
            this._loaded = true;
        }
        catch (e) {
            if (this.errorHandler && !(e instanceof CanceledPromiseError)) {
                this.errorHandler(e);
            }
        }
    }
    fetchRelatives() {
        var _a, _b, _c;
        const relativeMetadataValues = (_a = this.store.getState().map) === null || _a === void 0 ? void 0 : _a.metadata.filter((metadata) => { var _a; return metadata.key == ((_a = this.store.getState().configuration) === null || _a === void 0 ? void 0 : _a.relativePivotMetadata); }).flatMap((metadata) => metadata.values);
        if (relativeMetadataValues !== undefined && relativeMetadataValues.length > 0) {
            (_b = this.service) === null || _b === void 0 ? void 0 : _b.getRelativesForDocument((_c = this.store.getState().configuration) === null || _c === void 0 ? void 0 : _c.relativePivotMetadata, relativeMetadataValues).then((searchResults) => {
                const relatives = searchResults.results.flatMap((result) => result.entries);
                this.store.actions.relatives(relatives);
                this.updateClusteringHelper();
            });
        }
        else {
            this.updateClusteringHelper();
        }
    }
    updateClusteringHelper() {
        this.clusteringHelper = this.createClusteringHelper();
        if (this.clusteringHelper) {
            this.store.dispatchEvent(new CustomEvent("clustering-helper-created"));
        }
    }
    createClusteringHelper() {
        var _a, _b;
        if (this.clusteringHelperProvider) {
            return this.clusteringHelperProvider();
        }
        let relatives = this.store.getState().relatives;
        if (relatives) {
            let selectedResult = relatives.find((relative) => { var _a; return ((_a = relative.map) === null || _a === void 0 ? void 0 : _a.mapId) == this.store.getState().mapId; });
            let metadataDescriptors = (_b = (_a = ftAppInfoStore.getState().metadataConfiguration) === null || _a === void 0 ? void 0 : _a.descriptors) !== null && _b !== void 0 ? _b : [];
            return new ClusteringHelper(relatives, selectedResult, this.metadataForSwitchToRelatives, metadataDescriptors);
        }
        return undefined;
    }
    async navigateToRelativeTopicIfFound(topicPivotMetadataValue) {
        var _a, _b, _c;
        await this.loaded();
        const relativeTopicPivotMetadata = await waitFor(() => { var _a; return (_a = this.store.getState().configuration) === null || _a === void 0 ? void 0 : _a.relativeTopicPivotMetadata; });
        const mapId = this.store.getState().mapId;
        const service = await this.awaitService();
        const searchResults = await service.getRelativesForTopicInMap(mapId, relativeTopicPivotMetadata, [topicPivotMetadataValue]);
        if ((_c = (_b = (_a = searchResults === null || searchResults === void 0 ? void 0 : searchResults.results) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.entries) === null || _c === void 0 ? void 0 : _c[0]) {
            const targetTocId = searchResults.results[0].entries[0].topic.tocId;
            await this.navigateToTopic(targetTocId);
            return true;
        }
        else {
            await this.navigateToTopic("root");
            return false;
        }
    }
    setSearchInDocumentQuery(query) {
        this.store.actions.searchInDocumentQuery(query);
    }
    registerMetadataForSwitchToRelatives(id) {
        this.metadataForSwitchToRelatives.add(id);
        if (this.store.getState().relatives == undefined) {
            this.fetchRelatives();
        }
        else {
            this.updateClusteringHelper();
        }
    }
    unregisterMetadataForSwitchToRelatives(id) {
        this.metadataForSwitchToRelatives.delete(id);
        this.updateClusteringHelper();
    }
}
