class EnrichedTocBuilder {
    constructor(mapId, paginatedToc) {
        this.mapId = mapId;
        this.paginatedToc = paginatedToc;
        this.nodeByTocId = {};
        this.nodeByUrl = {};
        this.duplicatedPrettyUrls = new Set();
        this.pages = [];
        this.pageByTocId = {};
    }
    build() {
        return {
            paginationConfiguration: this.paginatedToc.configuration,
            toc: this.convertNodes(this.paginatedToc.paginatedToc, 1),
            pagesToc: this.extractPagesToc(this.paginatedToc.paginatedToc, 1),
            nodeByTocId: this.nodeByTocId,
            nodeByUrl: this.nodeByUrl,
            duplicatedPrettyUrls: this.duplicatedPrettyUrls,
            pages: this.pages,
            pageByTocId: this.pageByTocId,
        };
    }
    extractPagesToc(nodes, depth, parentTocId) {
        return nodes.flatMap(node => {
            if (node.pageConfiguration) {
                this.registerPage(this.buildPage(node, parentTocId));
            }
            if (node.tocId == null) {
                return this.extractPagesToc(node.children, depth, parentTocId);
            }
            return [
                {
                    tocId: node.tocId,
                    title: node.title,
                    depth,
                    children: this.extractPagesToc(node.children, depth + 1, node.tocId)
                }
            ];
        });
    }
    convertNodes(nodes, depth, parentTocId) {
        return nodes.flatMap(node => {
            var _a, _b, _c;
            if (node.tocId == null) {
                return [
                    ...this.convertSectionNodes((_a = node.pageToc) !== null && _a !== void 0 ? _a : [], depth, node.tocId),
                    ...this.convertNodes(node.children, depth, node.tocId)
                ];
            }
            let prettyUrl = (_b = node.prettyUrl) === null || _b === void 0 ? void 0 : _b.replace(/^\//, "");
            const enrichedNode = {
                tocId: node.tocId,
                contentId: node.contentId,
                title: node.title,
                hasRating: node.hasRating,
                origin: node.origin,
                prettyUrl,
                depth,
                parentTocId: parentTocId,
                children: []
            };
            this.register(enrichedNode);
            const sectionChildren = this.convertSectionNodes((_c = node.pageToc) !== null && _c !== void 0 ? _c : [], depth + 1, node.tocId);
            const childrenWithPages = this.convertNodes(node.children, depth + 1, enrichedNode.tocId);
            enrichedNode.children = [
                ...sectionChildren,
                ...childrenWithPages
            ];
            return [enrichedNode];
        });
    }
    convertSectionNodes(nodes, depth, parentTocId) {
        return nodes.flatMap(node => {
            var _a;
            let prettyUrl = (_a = node.prettyUrl) === null || _a === void 0 ? void 0 : _a.replace(/^\//, "");
            const enrichedNode = {
                ...node,
                prettyUrl,
                depth,
                parentTocId,
                children: []
            };
            this.register(enrichedNode);
            enrichedNode.children = this.convertSectionNodes(node.children, depth + 1, node.tocId);
            return [enrichedNode];
        });
    }
    buildPage(node, parentTocId) {
        var _a, _b, _c, _d, _e, _f;
        const page = {
            number: this.pages.length + 1,
            title: (_a = node.title) !== null && _a !== void 0 ? _a : "",
            rootTocId: (_b = node.tocId) !== null && _b !== void 0 ? _b : "root",
            toc: ((_c = node.pageToc) !== null && _c !== void 0 ? _c : []).map((n) => this.nodeByTocId[n.tocId]),
            topics: [],
            hiddenTopics: [],
            breadcrumb: parentTocId ? this.buildBreadcrumb(this.nodeByTocId[parentTocId]) : []
        };
        const add = (tocId, visible) => (visible ? page.topics : page.hiddenTopics).push(tocId);
        const parentsVisibility = (_e = (_d = node.pageConfiguration) === null || _d === void 0 ? void 0 : _d.parentsVisibility) !== null && _e !== void 0 ? _e : [];
        page.breadcrumb.forEach((tocId, index) => {
            add(tocId, parentsVisibility[index]);
        });
        if (node.tocId) {
            add(node.tocId, node.pageConfiguration.isVisible);
        }
        page.topics.push(...this.tocIds((_f = node.pageToc) !== null && _f !== void 0 ? _f : []));
        return page;
    }
    registerPage(page) {
        for (let tocId of [...page.topics, ...page.hiddenTopics]) {
            if (!this.pageByTocId[tocId]) {
                this.pageByTocId[tocId] = page;
            }
        }
        this.pages.push(page);
    }
    tocIds(nodes) {
        return nodes.flatMap(node => [node.tocId, ...this.tocIds(node.children)]);
    }
    buildBreadcrumb(node) {
        const breadcrumb = [node.tocId];
        while (node === null || node === void 0 ? void 0 : node.parentTocId) {
            breadcrumb.unshift(node.parentTocId);
            node = this.nodeByTocId[node.parentTocId];
        }
        return breadcrumb;
    }
    register(enrichedNode) {
        if (enrichedNode.prettyUrl) {
            if (this.nodeByUrl[enrichedNode.prettyUrl] == null) {
                this.nodeByUrl[enrichedNode.prettyUrl] = enrichedNode;
            }
            else {
                this.duplicatedPrettyUrls.add(enrichedNode.prettyUrl);
            }
        }
        this.nodeByUrl[`${this.mapId}/${enrichedNode.tocId}`] = enrichedNode;
        this.nodeByTocId[enrichedNode.tocId] = enrichedNode;
    }
}
export class FtReaderConverter {
    convertPaginatedToc(mapId, paginatedToc) {
        return new EnrichedTocBuilder(mapId, paginatedToc).build();
    }
}
