import { FtUserRole, userHasRole } from "@fluid-topics/public-api";
import { FtReaderFeatures } from "../model";
import { ftAppInfoStore, FtServiceWithCache } from "@fluid-topics/ft-app-context";
export class FtReaderService extends FtServiceWithCache {
    get mapId() {
        return this._mapId;
    }
    set mapId(value) {
        if (value !== this._mapId) {
            this.clear();
        }
        this._mapId = value;
    }
    constructor(converter, overrideApi, mapId) {
        super(false, overrideApi);
        this.converter = converter;
        this._mapId = mapId;
        this.initCache();
    }
    async withMapId(provider) {
        return this._mapId ? provider(this._mapId) : undefined;
    }
    initCache() {
        this.cache.registerFinal("configuration", () => this.api.getReaderConfiguration());
        this.cache.register("map", () => this.withMapId(mapId => this.fetchMap(mapId)));
        this.cache.register("enrichedToc", () => this.withMapId(mapId => this.fetchPages(mapId).then(toc => this.converter.convertPaginatedToc(mapId, toc))));
        this.cache.register("ratingSummary", () => {
            var _a, _b;
            const userHaveRatingRole = userHasRole((_b = (_a = ftAppInfoStore.getState().session) === null || _a === void 0 ? void 0 : _a.profile.roles) !== null && _b !== void 0 ? _b : [], "RATING_USER");
            return userHaveRatingRole ? this.withMapId(mapId => this.fetchMapRating(mapId)) : Promise.resolve();
        });
    }
    clear() {
        this.cache.clearAll();
    }
    getMap() {
        return this.cache.get("map");
    }
    getMapNow() {
        return this.cache.getNow("map");
    }
    getConfiguration() {
        return this.cache.get("configuration");
    }
    getEnrichedToc() {
        return this.cache.get("enrichedToc");
    }
    get enrichedToc() {
        return this.cache.getNow("enrichedToc");
    }
    async getPaginationConfiguration() {
        const enrichedToc = await this.getEnrichedToc();
        return enrichedToc.paginationConfiguration;
    }
    async getToc() {
        const enrichedToc = await this.getEnrichedToc();
        return enrichedToc.toc;
    }
    async getPagesToc() {
        const enrichedToc = await this.getEnrichedToc();
        return enrichedToc.pagesToc;
    }
    async getTocNode(tocId) {
        const enrichedToc = await this.getEnrichedToc();
        return enrichedToc.nodeByTocId[tocId];
    }
    getTocNodeNow(tocId) {
        var _a;
        return (_a = this.enrichedToc) === null || _a === void 0 ? void 0 : _a.nodeByTocId[tocId];
    }
    getTocNodeNowByUrl(prettyUrlOrMapIdTocId) {
        var _a, _b;
        let map = this.getMapNow();
        if ((map === null || map === void 0 ? void 0 : map.prettyUrl) == prettyUrlOrMapIdTocId || `${map === null || map === void 0 ? void 0 : map.id}/root` === prettyUrlOrMapIdTocId) {
            return (_a = this.enrichedToc) === null || _a === void 0 ? void 0 : _a.toc[0];
        }
        return (_b = this.enrichedToc) === null || _b === void 0 ? void 0 : _b.nodeByUrl[prettyUrlOrMapIdTocId];
    }
    async getPage(number) {
        const enrichedToc = await this.getEnrichedToc();
        return enrichedToc.pages[number - 1] || enrichedToc.pages[0];
    }
    async getPageByTocId(tocId) {
        const enrichedToc = await this.getEnrichedToc();
        return enrichedToc.pageByTocId[tocId] || enrichedToc.pages[0];
    }
    getTopicInfo(tocNode) {
        return this.cache.get("topic-" + tocNode.tocId, () => this.fetchTopicMetadata(tocNode));
    }
    getTopicContent(tocNode) {
        return this.cache.get("topic-content-" + tocNode.tocId, () => this.fetchTopicHTMLContent(tocNode));
    }
    getAttachments() {
        return this.cache.get("map-attachments", () => this.fetchAttachments());
    }
    getRelativesForDocument(metadataPivotKey, metadataPivotValues) {
        return this.cache.get("document-relatives-" + metadataPivotKey + "-" + metadataPivotValues.join("-"), () => this.fetchRelativesForDocument(metadataPivotKey, metadataPivotValues));
    }
    getRelativesForTopicInMap(mapId, metadataPivotKey, metadataPivotValues) {
        return this.cache.get("topic-relatives-" + [mapId, metadataPivotKey, ...metadataPivotValues].join("-"), () => this.fetchRelativesForTopicInMap(mapId, metadataPivotKey, metadataPivotValues));
    }
    getAttachmentDownloadLink(id) {
        return this.makeAbsolute(`${this.api.endpoints.khub.maps.mapId(this.mapId).attachments.attachmentId(id).content}?download=true`);
    }
    getLink(tocId, pageNumber, section) {
        const navigationData = this.getNavigationData(tocId, pageNumber, section);
        const path = this.getReaderLinkPrefix() + (navigationData.prettyUrl ? navigationData.prettyUrl : `${navigationData.mapId}/${navigationData.tocId}`);
        const url = new URL(path, this.api.tenantBaseUrl);
        if (navigationData.page != null) {
            url.searchParams.set("page", `${navigationData.page}`);
        }
        if (navigationData.prettyUrlDuplicated) {
            url.searchParams.set("tocId", navigationData.tocId);
        }
        if (navigationData.section != null) {
            url.searchParams.set("section", navigationData.section);
        }
        return url.href;
    }
    getNavigationData(tocId, pageNumber, section) {
        var _a, _b;
        tocId = tocId || "root";
        const enrichedToc = this.enrichedToc;
        const page = enrichedToc === null || enrichedToc === void 0 ? void 0 : enrichedToc.pageByTocId[tocId];
        const node = enrichedToc === null || enrichedToc === void 0 ? void 0 : enrichedToc.nodeByTocId[tocId];
        const prettyUrl = node === null || node === void 0 ? void 0 : node.prettyUrl;
        return {
            mapId: this.mapId,
            tocId,
            topicTitle: (_a = node === null || node === void 0 ? void 0 : node.title) !== null && _a !== void 0 ? _a : "",
            page: pageNumber && page && page.number !== pageNumber ? pageNumber : undefined,
            prettyUrl: prettyUrl != null ? prettyUrl.replace(/^r\//, "") : undefined,
            prettyUrlDuplicated: (_b = enrichedToc === null || enrichedToc === void 0 ? void 0 : enrichedToc.duplicatedPrettyUrls.has(prettyUrl !== null && prettyUrl !== void 0 ? prettyUrl : "")) !== null && _b !== void 0 ? _b : false,
            section: section
        };
    }
    async getAccurateNavigationData(tocId, pageNumber, section) {
        await this.getEnrichedToc();
        return this.getNavigationData(tocId, pageNumber, section);
    }
    getRatingSummary() {
        return this.cache.get("ratingSummary");
    }
    reloadRating() {
        this.cache.clear("ratingSummary");
        return this.getRatingSummary();
    }
    getMapRating() {
        return this.getRatingSummary().then(rm => rm === null || rm === void 0 ? void 0 : rm.publication);
    }
    getTopicRating(tocId) {
        return this.getRatingSummary().then(rm => {
            var _a;
            return rm ? {
                type: rm.topics.type,
                rating: (_a = rm.topics) === null || _a === void 0 ? void 0 : _a.ratings.find(t => t.tocId === tocId)
            } : undefined;
        });
    }
    rateMap(type, value) {
        return this.makeRateMapRequest(type, value).then(() => this.clearRatingIfNecessary());
    }
    unrateMap() {
        return this.makeUnrateMapRequest().then(() => this.clearRatingIfNecessary());
    }
    sendMapFeedback(message, from) {
        return this.makeMapFeedbackRequest(message, from);
    }
    rateTopic(tocId, type, value) {
        return this.makeRateTopicRequest(tocId, type, value).then(() => this.clearRatingIfNecessary());
    }
    unrateTopic(tocId) {
        return this.makeUnrateTopicRequest(tocId).then(() => this.clearRatingIfNecessary());
    }
    async clearRatingIfNecessary() {
        const session = ftAppInfoStore.getState().session;
        if (session === null || session === void 0 ? void 0 : session.sessionAuthenticated) {
            this.cache.clear("ratingSummary");
        }
    }
    sendTopicFeedback(tocId, message, from) {
        return this.makeTopicFeedbackRequest(tocId, message, from);
    }
    fontsStylesheet() {
        return this.api.makeAbsolute(this.api.endpoints.stylesheets.fonts);
    }
    themeStylesheet() {
        return this.api.makeAbsolute(this.api.endpoints.stylesheets.theme);
    }
    buildBreadcrumb(tocId) {
        let target = this.getTocNodeNow(tocId);
        let breadcrumb = [{ tocId: target.tocId, title: target.title }];
        let parent = this.getTocNodeNow(target.parentTocId);
        while (parent) {
            breadcrumb.unshift({ tocId: parent.tocId, title: parent.title });
            parent = this.getTocNodeNow(parent.parentTocId);
        }
        return breadcrumb;
    }
    isFeatureAccessible(feature, { session }) {
        var _a;
        if (!this.isFeatureAvailable(feature)) {
            return false;
        }
        switch (feature) {
            case FtReaderFeatures.FEEDBACK:
                return userHasRole(session !== null && session !== void 0 ? session : [], FtUserRole.FEEDBACK_USER);
            case FtReaderFeatures.RATING:
                return userHasRole(session !== null && session !== void 0 ? session : [], FtUserRole.RATING_USER);
            case FtReaderFeatures.PRINT:
                return userHasRole(session !== null && session !== void 0 ? session : [], FtUserRole.PRINT_USER);
            case FtReaderFeatures.BOOKMARK:
                return !!(session === null || session === void 0 ? void 0 : session.sessionAuthenticated);
            case FtReaderFeatures.COLLECTIONS:
                return userHasRole(session !== null && session !== void 0 ? session : [], FtUserRole.COLLECTION_USER);
            case FtReaderFeatures.PERSONAL_BOOKS:
                return userHasRole(session !== null && session !== void 0 ? session : [], FtUserRole.PERSONAL_BOOK_USER) && !((_a = this.getMapNow()) === null || _a === void 0 ? void 0 : _a.pbkConfidential);
            default:
                return false;
        }
    }
    makeAbsolute(endpoint) {
        return this.api.makeAbsolute(endpoint);
    }
}
